﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Libra.Attributes;


namespace Libra.Extensions
{
    /// <summary>Тип:
    /// представляет собой набор методов расширения для работы с перечислениями 
    /// </summary>
    public static class EnumExtensions
    {
		/// <summary>Метод:
		/// возвращает имя заложенное в элемент перечисления атрибутом ViewAttribute, а в отстуствии атрибута возвращает просто название поля
		/// </summary>
		public static string GetName(this Enum @enum)
        {
            if (@enum == null) return "null";
            
            var fieldInfo = @enum.GetType().GetTypeInfo().GetDeclaredField(@enum.ToString());
            var attribute = ((ViewAttribute[])fieldInfo.GetCustomAttributes(typeof(ViewAttribute), false)).FirstOrDefault();

            if (!string.IsNullOrEmpty(attribute?.Name))
				return attribute.Name;

            return @enum.ToString();
        }



		/// <summary>Метод:
		/// возвращает описание заложенное в элемент перечисления атрибутом ViewAttribute,
		/// либо же имя заложенное в элемент перечисления этим атрибутом,
		/// а в отстуствии атрибута возвращает просто название поля
		/// </summary>
		public static string GetDescriptionOrName(this Enum @enum)
        {
	        if (@enum == null) return "null";

	        var fieldInfo = @enum.GetType().GetTypeInfo().GetDeclaredField(@enum.ToString());
	        var attribute = ((ViewAttribute[])fieldInfo.GetCustomAttributes(typeof(ViewAttribute), false)).FirstOrDefault();

	        if (attribute != null)
	        {
		        if (!string.IsNullOrEmpty(attribute.Description))
			        return attribute.Description;

				if (!string.IsNullOrEmpty(attribute.Name))
					return attribute.Name;
			}

	        return @enum.ToString();
		}



        /// <summary>Метод:
        /// позволяет получить все возможные значения указанного перечисления
        /// </summary>
        /// <typeparam name="T">тип перечисления значения которого необходимо получить</typeparam>
        /// <returns>возвращает перечисление всех значений заданного перечисления</returns>
        public static IEnumerable<T> GetValues<T>()
        {
            return Enum.GetValues(typeof(T)).Cast<T>();
        }
    }
}