﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Libra.Primitives;
using Libra.Utilities;


namespace Libra
{
    public class MainEntry
    {
        //      #fields         ---     ---     ---

        private static object Locker = new object();
        private readonly IDictionary<ChartSourceIdentifier, int> _counters = new Dictionary<ChartSourceIdentifier, int>();

        
        //      #properties     ---     ---     ---

        public static MainEntry Instance { get; } = new MainEntry();


        //      #methods        ---     ---     ---

        [DllExport(CallingConvention.StdCall)]
        public static int Initialize([MarshalAs(UnmanagedType.LPWStr)] string symbol, Timeframe timeframe)
        {
            SimpleLogger.Instance.Write(nameof(Initialize), $"symbol: {symbol}, timeframe: {timeframe}");

            lock (Locker)
            {
                try
                {
                    var key = new ChartSourceIdentifier(symbol, timeframe);
                    if (Instance._counters.ContainsKey(key))
                    {
                        int value = Instance._counters[key];
                        value++;
                        Instance._counters[key] = value;
                        return value;
                    }

                    Instance._counters[key] = 1;
                }
                catch (Exception ex)
                {
                    SimpleLogger.Instance.Write(nameof(Initialize), ex);
                }
            }

            return 1;
        }


        [DllExport(CallingConvention.StdCall)]
        public static void WriteToLog([MarshalAs(UnmanagedType.LPWStr)] string tag, [MarshalAs(UnmanagedType.LPWStr)] string record)
        {
            SimpleLogger.Instance.Write(tag, record);
        }

        
        [DllExport(CallingConvention.StdCall)]
        public static void Wait(int milliseconds)
        {
            Thread.Sleep(milliseconds);
        }


        [DllExport(CallingConvention.StdCall)]
        public static string GetReport([MarshalAs(UnmanagedType.LPWStr)] string symbol, Timeframe timeframe)
        {
            SimpleLogger.Instance.Write(nameof(GetReport), $"symbol: {symbol}, timeframe: {timeframe}");
            StringBuilder report = new StringBuilder();
            
            lock (Locker)
            {
                try
                {
                    foreach (var counter in Instance._counters.Where(c => c.Key.Symbol == symbol).OrderBy(c => c.Key.Timeframe))
                    {
                        report.AppendFormat("{0} {1}: {2}\n", counter.Key.Symbol, counter.Key.Timeframe, counter.Value);
                    }
                }
                catch (Exception ex)
                {
                    SimpleLogger.Instance.Write(nameof(GetReport), ex);
                }
            }

            return report.ToString();
        }


        [DllExport(CallingConvention.StdCall)]
        public static void FillReport([MarshalAs(UnmanagedType.LPWStr)] string symbol, Timeframe timeframe, [In, Out, MarshalAs(UnmanagedType.LPWStr)] StringBuilder report)
        {
            SimpleLogger.Instance.Write(nameof(FillReport), $"symbol: {symbol}, timeframe: {timeframe}, report {(report == null ? "=" : "!")}= null");
            if (report == null)
                return;
            
            lock (Locker)
            {
                try
                {
                    foreach (var counter in Instance._counters.Where(c => c.Key.Symbol == symbol).OrderBy(c => c.Key.Timeframe))
                    {
                        report.AppendFormat("{0} {1}: {2}\n", counter.Key.Symbol, counter.Key.Timeframe, counter.Value);
                    }
                }
                catch (Exception ex)
                {
                    SimpleLogger.Instance.Write(nameof(FillReport), ex);
                }
            }

            SimpleLogger.Instance.Write(nameof(FillReport), "End!");
        }
    }
}