﻿using System;


namespace Libra.Primitives
{
    public class ChartSourceIdentifier : IEquatable<ChartSourceIdentifier>
    {
        //      #constructors   ---     ---     ---

        public ChartSourceIdentifier(string symbol, Timeframe timeframe)
        {
            if (!Symbols.Validate(symbol))
                throw new ArgumentException($"{nameof(symbol)} is not a proper value");

            Symbol = symbol;
            Timeframe = timeframe;
        }


        //      #properties     ---     ---     ---

        public string Symbol { get; }

        public Timeframe Timeframe { get; }


        //      #methods        ---     ---     ---

        public static bool operator ==(ChartSourceIdentifier left, (string, Timeframe) right) => !ReferenceEquals(left, null) && right.Item1.Equals(left.Symbol) && right.Item2.Equals(left.Timeframe);

        public static bool operator !=(ChartSourceIdentifier left, (string, Timeframe) right) => !(left == right);
        
        public static bool operator ==(ChartSourceIdentifier left, ChartSourceIdentifier right) => Equals(left, right);
        
        public static bool operator !=(ChartSourceIdentifier left, ChartSourceIdentifier right) => !Equals(left, right);
        
        public static bool Equals(ChartSourceIdentifier left, ChartSourceIdentifier right) => !ReferenceEquals(left, null) && left.Equals(right);


        public bool Equals(ChartSourceIdentifier other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return string.Equals(Symbol, other.Symbol) && Timeframe == other.Timeframe;
        }


        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((ChartSourceIdentifier) obj);
        }
        

        public override int GetHashCode()
        {
            unchecked
            {
                return ((Symbol != null ? Symbol.GetHashCode() : 0) * 397) ^ (int) Timeframe;
            }
        }
    }
}