﻿using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Libra.Extensions;


namespace Libra
{
    public static class Symbols
    {
        //      #fields         ---     ---     ---

        private static readonly List<string> SymbolsList;


        //      #constructors   ---     ---     ---

        static Symbols()
        {
            SymbolsList = typeof(Symbols).GetAllPublicConstantValues<string>();
        }


        //      #methods        ---     ---     ---

        public static bool Validate(string symbol)
        {
            if (string.IsNullOrWhiteSpace(symbol))
                return false;

            return SymbolsList.Any(prefix => symbol.StartsWith(prefix, true, CultureInfo.InvariantCulture));
        }


        public static (string symbol, string suffix)? Disintegrate(string symbol)
        {
            if (string.IsNullOrWhiteSpace(symbol))
                return null;

            foreach (string symbolPrefix in SymbolsList)
                if (symbol.StartsWith(symbolPrefix, true, CultureInfo.InvariantCulture))
                {
                    if (symbol.Length > symbolPrefix.Length)
                        return (symbolPrefix, symbol.Substring(symbolPrefix.Length));

                    return (symbolPrefix, null);
                }

            return null;
        }


        //      ---     ---     ---     ---     ---

        // Australian Dollar / Canadian Dollar
        public const string AudCad = "AUDCAD";

        // Australian Dollar / Swiss Franc
        public const string AudChf = "AUDCHF";

        // Australian Dollar / Japanese Yen
        public const string AudJpy = "AUDJPY";

        // Australian Dollar / New Zealand Dollar
        public const string AudNzd = "AUDNZD";

        // Australian Dollar / US Dollar
        public const string AudUsd = "AUDUSD";

        // Canadian Dollar / Swiss Franc
        public const string CadChf = "CADCHF";

        // Canadian Dollar / Japanese Yen
        public const string CadJpy = "CADJPY";

        // Swiss Franc / Japanese Yen
        public const string ChfJpy = "CHFJPY";

        // Euro / Australian Dollar
        public const string EurAud = "EURAUD";

        // Euro / Canadian Dollar
        public const string EurCad = "EURCAD";

        // Euro / Swiss Franc
        public const string EurChf = "EURCHF";

        // Euro / Danish Krone
        public const string EurDkk = "EURDKK";

        // Euro / Great Britain Pound
        public const string EurGbp = "EURGBP";

        // Euro / Japanese Yen
        public const string EurJpy = "EURJPY";

        // Euro / Norwegian Krone
        public const string EurNok = "EURNOK";

        // Euro / New Zealand Dollar
        public const string EurNzd = "EURNZD";

        // Euro / Russian Ruble
        public const string EurRub = "EURRUB";

        // Euro / Swedish Krone
        public const string EurSek = "EURSEK";

        // Euro / Singapore Dollar
        public const string EurSgd = "EURSGD";

        // Euro / Turkish Lira
        public const string EurTry = "EURTRY";

        // Euro / US Dollar
        public const string EurUsd = "EURUSD";

        // Great British Pound / Australian Dollar
        public const string GbpAud = "GBPAUD";

        // Great British Pound / Canadian Dollar
        public const string GbpCad = "GBPCAD";

        // Great Britain Pound / Swiss Franc
        public const string GbpChf = "GBPCHF";

        // Great Britain Pound / Japanese Yen
        public const string GbpJpy = "GBPJPY";

        // Great Britain Pound / New Zealand Dollar
        public const string GbpNzd = "GBPNZD";

        // Great British Pound / Singapore Dollar
        public const string GbpSgd = "GBPSGD";

        // Great Britain Pound / Turkish Lira
        public const string GbpTry = "GBPTRY";

        // Great Britain Pound / US Dollar
        public const string GbpUsd = "GBPUSD";

        // New Zealand Dollar / Canadian Dollar
        public const string NzdCad = "NZDCAD";

        // New Zealand Dollar / Swiss Franc
        public const string NzdChf = "NZDCHF";

        // New Zealand Dollar / Japanese Yen
        public const string NzdJpy = "NZDJPY";

        // New Zealand Dollar / Singapore Dollar
        public const string NzdSgd = "NZDSGD";

        // New Zealand Dollar / US Dollar
        public const string NzdUsd = "NZDUSD";

        // Turkish Lira / Japanese Yen
        public const string TryJpy = "TRYJPY";

        // US Dollar / Canadian Dollar
        public const string UsdCad = "USDCAD";

        // US Dollar / Swiss Franc
        public const string UsdChf = "USDCHF";

        // US Dollar / China Offshore Spot
        public const string UsdCnh = "USDCNH";

        // US Dollar / Danish Krone
        public const string UsdDkk = "USDDKK";

        // US Dollar / Japanese Yen
        public const string UsdJpy = "USDJPY";

        // US Dollar / Mexican Peso
        public const string UsdMxn = "USDMXN";

        // US Dollar / Norwegian Krone
        public const string UsdNok = "USDNOK";

        // US Dollar / Polish Zloty
        public const string UsdPln = "USDPLN";

        // US Dollar / Russian Ruble
        public const string UsdRub = "USDRUB";

        // US Dollar / Swedish Krone
        public const string UsdSek = "USDSEK";

        // US Dollar / Singapore Dollar
        public const string UsdSgd = "USDSGD";

        // US Dollar / Turkish Lira
        public const string UsdTry = "USDTRY";

        // US Dollar / South African Rand
        public const string UsdZar = "USDZAR";
    }
}