﻿using System;
using System.IO;
using System.Text;
using Libra.Utilities.Interfaces;


namespace Libra.Utilities
{
    public class SimpleLogger : ILogger
    {
        //      #properties     ---     ---     ---

        public static ILogger Instance { get; private set; } = new SimpleLogger();

        public string LogsDirectory => Settings.LogsDirectory;


        //      #methods        ---     ---     ---

        public void Write<T>(string tag, T message, uint newLineCountAtEnding = 1)
        {
            WriteToFile(FormMessage(tag, message?.ToString(), newLineCountAtEnding));
        }

        public void Write(string tag, string message, Exception ex)
        {
            WriteToFile(FormMessage(tag, $"{message}{Environment.NewLine}Exception: {ex}", 1));
        }


        //      #utility methods --     ---     ---

        private static string FormMessage(string tag, string message, uint newLineCountAtEnding)
        {
            var stringBuilder = new StringBuilder();

            stringBuilder.AppendFormat("{0} ## {1:dd.MM.yyyy HH:mm:ss}:{3} >> {2}{3}", tag, DateTime.Now, message ?? "NULL", Environment.NewLine);

            for (int i = 0; i < newLineCountAtEnding; i++)
                stringBuilder.AppendLine();

            return stringBuilder.ToString();
        }
        

        private static void WriteToFile(string message)
        {
            if (!Directory.Exists(Settings.LogsDirectory))
                Directory.CreateDirectory(Settings.LogsDirectory);

            if (File.Exists(Settings.LogFilePath))
                File.AppendAllText(Settings.LogFilePath, message, Encoding.UTF8);
            else
                File.WriteAllText(Settings.LogFilePath, message, Encoding.UTF8);
        }


        //      #internal types ---     ---     ---

        public static class Settings
        {
            //      #constructors   ---     ---     ---

            static Settings()
            {
                LogsDirectory = Path.Combine(Environment.CurrentDirectory, "Logs");
                LogFilePath = Path.Combine(LogsDirectory, $"{nameof(Libra)}_{DateTime.Now:yyyy-MMMM-dd__HH-mm}_log.txt");
            }

            
            //      #properties     ---     ---     ---

            public static string LogsDirectory { get; }

            public static string LogFilePath { get; }
        }
    }
}